/**
 * @file
 * Defines the behavior of the Message Banner.
 */

(function ($, Drupal, drupalSettings) {
  /**
   * Message banner behavior.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   If the banner is enabled, check the user's cookie to see if it should be
   *   displayed to them.
   */
  Drupal.behaviors.messageBanner = {
    /**
     * Attach the message banner behavior.
     *
     * @param {Object} context
     *   The DOM context.
     */
    attach(context) {
      const settings = drupalSettings.message_banner;
      const enabled = settings.banner_enabled;

      if (!enabled) {
        return;
      }

      // Check to see if cookies are enabled.
      if (!Drupal.messageBanner.cookiesEnabled()) {
        return;
      }

      const currentUserValue = Drupal.messageBanner.getCurrentUserValue();
      const currentConfigValue = settings.banner_timestamp;

      // Check if the current status matches the banner timestamp. If there is
      // no match, create the banner and set a cookie when the banner is closed.
      if (currentUserValue !== currentConfigValue) {
        Drupal.messageBanner.createBanner(settings.attention_banner_text, context);
        $('.close-banner', context).click(function () {
          $(this).closest('.message-banner').slideUp(200, function () {
            $(this).remove();
          });
          Drupal.messageBanner.setNewUserValue(currentConfigValue);
        });
      }
    },
  };

  Drupal.messageBanner = {};

  /**
   * Create a new cookie banner using the HTML from a WYSIWYG.
   *
   * @param string html
   *   The HTML from the WYSIWYG, entered by admins in the config form.
   */
  Drupal.messageBanner.createBanner = function (html, context) {
    const $banner = $(html);
    const $cookieBanner = $('#message-banner', context);
    if ($cookieBanner.length) {
      $banner.insertAfter($cookieBanner);
    } else {
      $banner.prependTo($('body', context));
    }
  };

  /**
   * Check to see if a cookie already exists.
   *
   * @return
   *   Cookie value.
   */
  Drupal.messageBanner.getCurrentUserValue = function () {
    const result = Drupal.messageBanner.getCookie('banner-agreed');
    return parseInt(result);
  };

  /**
   * Change the cookie status.
   *
   * @param currentConfigValues
   *   The time stamp value currently stored in drupal settings config.
   */
  Drupal.messageBanner.setNewUserValue = function (currentConfigValue) {
    const currentUserValue = Drupal.messageBanner.getCurrentUserValue();

    if (currentConfigValue === currentUserValue) {
      return;
    }

    const cookieExpiryDate = new Date();
    cookieExpiryDate.setDate(cookieExpiryDate.getDate() + 100);

    const cookie = `banner-agreed=${currentConfigValue};expires=${cookieExpiryDate.toUTCString()};path=${drupalSettings.path.baseUrl}`;

    document.cookie = cookie;
  };

  /**
   * Verbatim copy of Drupal.comment.getCookie().
   *
   * @param string name
   *   The name of the cookie to retrieve from the user's computer.
   *
   * @return
   *   The value of the cookie.
   */
  Drupal.messageBanner.getCookie = function (name) {
    const search = `${name}=`;
    let returnValue = '0';

    if (document.cookie.length > 0) {
      let offset = document.cookie.indexOf(search);
      if (offset !== -1) {
        offset += search.length;
        let end = document.cookie.indexOf(';', offset);
        if (end === -1) {
          end = document.cookie.length;
        }
        returnValue = decodeURIComponent(document.cookie.substring(offset, end).replace(/\+/g, '%20'));
      }
    }
    return returnValue;
  };

  /**
   * Test to see if the cookie is enabled.
   *
   * @return
   *   The value of cookieEnabled.
   */
  Drupal.messageBanner.cookiesEnabled = function () {
    let { cookieEnabled } = navigator;
    if (typeof navigator.cookieEnabled === 'undefined' && !cookieEnabled) {
      document.cookie = 'testcookie';
      cookieEnabled = (document.cookie.indexOf('testcookie') !== -1);
    }
    return (cookieEnabled);
  };
}(jQuery, Drupal, drupalSettings));
